/* P3PEvaluator.h - header file for P3PEvaluator.c

*/

#ifndef P3PEVALUATOR_H
#define P3PEVALUATOR_H

/* error codes - these are what a_errno returns */
#define A_OK                    0
#define A_CHAR_CONVERSION_ERROR 1
#define A_NO_MEMORY             2
#define A_NONEXISTENT_FILE      3
#define A_XML_PARSER_ERROR      4
#define A_APPEL_PARSING_ERROR   5
#define A_P3P_PARSING_ERROR     6
#define A_NULL_POINTER_ERROR    7
#define A_BEHAVIOR_REQUEST      8

/* result codes when both files are valid and no other errors encountered -
   these codes are used in the flags field of APPEL_RULE_T */
#define P3PMATCH_REQUEST 1
#define P3PMATCH_LIMITED 2
#define P3PMATCH_BLOCK   3
/* to be "OR"'d to other values */
#define P3PMATCH_PROMPT  0x8000
#define VARIABLE_CATEGORY	"VARIABLE_CATEGORY"

/* types */

#include "GString.h"
#include "GContainer.h"
#include "GURL.h"
#include "XmlNode.h"


class DisputeInfo: public GPEnabled
{
public:
	GString ImageURL;
	GString ServiceURL;
	GString ShortDescription;
	GString LongDescription;
	GString ImgAltText;
};

class DataInfo: public GPEnabled
{
public:
	DataInfo(){};
	DataInfo(GString data):DataCollected(data){};
	GString DataCollected;
	GMap<GString, GString> CatInfo;
};

class StatementInfo: public GPEnabled
{
public:
	GString ExtensionName;
	GMap<GString, DataInfo> InfoCollected;
	GMap<GString, GString> Purposes;
	GMap<GString, GString> Recipients;
	GString Consequence;
};


class Result 
{
public:
	GURL resultforURL;
	GURL appliedPolicyURL;
	enum P3PMatch{ UNKNOWN=-1, MATCH, UNMATCH};
	P3PMatch Match;
	Result(GURL forURL):resultforURL(forURL), Flags(0), Match(UNKNOWN), Test(false) {}
	Result():Flags(0), Match(UNKNOWN), Test(false) {}


	///policy info
	GString CreatedBy;
    GString CreatedOn;
    GString Persona;
	GString DiscURI;
	GString OptURI;
	GString Expiry;
	bool Test;


	////entity info
	GString BusinessName;
	GString BusinessPostalName;
	GString BusinessPostalStreet;
	GString BusinessPostalCity;
	GString BusinessPostalState;
	GString BusinessPostalCode;
	GString BusinessPostalCountry;
	GString BusinessTelephoneIntCode;
	GString BusinessTelephoneLocCode;
	GString BusinessTelephoneNumber;
	GString BusinessTelephoneExt;
	GString BusinessTelephoneComment;
	GString BusinessFaxIntCode;
	GString BusinessFaxLocCode;
	GString BusinessFaxNumber;
	GString BusinessFaxExt;
	GString BusinessFaxComment;
	GString BusinessOnlineEmail;
	GString BusinessOnlineURI;

	///disputes info
	GPList<DisputeInfo> Disputes;

	///statement info
	GPList<StatementInfo> Statements;

	/// info collected
/*	GMap<GString, GString> InfoCollected;
	GMap<GString, GString> Purposes;
	GMap<GString, GString> Recipients;
*/	GString Access;

	///mismatches and prompts
    GList<GString> MisMatches;
    GList<GString> PromptMsgs;
    unsigned Flags;             /* request, limited, block, prompt */
    
};


class P3PEvaluator
{
public:

static int P3PEvaluator_Match(const GString &appel_filename, const GString &p3p_filename, 
					   const GURL &policyurl, const GString &policyname,
                                        Result &result);

static int P3PEvaluator_Match_Memory(const char *AppelBuffer, int appelLength, 
							  const char *P3PBuffer, int P3PLength, 
							  const GURL &policyurl, const GString &policyname,
                                            Result &result);
static int P3PEvaluator::SimplifyDataDefs(GP<XmlNode> dataschema);

private:
static GString P3PEvaluator::PrepareNewDataDefName(const char *defname, const char *refstructname, const char *matchstructname);
static bool P3PEvaluator::isDataDefHasCategories(GP<XmlNode> &datadef);
static GP<XmlNode> P3PEvaluator::FindDataDef(GP<XmlNode> scheme, GString name );
static bool P3PEvaluator::FindAttachSupersetDataDef(GP<XmlNode> nodeparent, GP<XmlNode> node, GP<XmlNode> scheme, GString name, GString base );
static GP<XmlNode> P3PEvaluator::FindDataSchemaAndSimplify(GP<XmlNode> p3ptree);
static void P3PEvaluator::AttachDataDef(GP<XmlNode> node, GP<XmlNode> datadef);
static GP<XmlNode> P3PEvaluator::SimplifyPolicyTree(GP<XmlNode> p3ptree, const GURL &policyurl, GString policyname);
static bool P3PEvaluator::IsExtensionRequired(GP<XmlNode> p3p_tree);
static bool P3PEvaluator::CheckPolicyName(GP<XmlNode> policy, const GString &policyname);
static GP<XmlNode> P3PEvaluator::FindNamedPolicy(GP<XmlNode> policiesnode, const GString &policyname);
static int P3PEvaluator::P3PEvalutor_Match(GP<XmlNode> appel_tree, GP<XmlNode> p3p_tree, const GURL &policyurl, 
					  const GString &policyname, Result &result);
static bool P3PEvaluator::IsEntityMissingRequiredFields(GP<XmlNode> entitytree);
static bool P3PEvaluator::IsStatementMissingRequiredFields(GP<XmlNode> statementtree);
static bool P3PEvaluator::IsAccessMissingRequiredFields(GP<XmlNode> accesstree);
static bool P3PEvaluator::IsPolicyMissingRequiredFields(GP<XmlNode> policytree);

};



#endif

/* END OF FILE P3PEvaluator.H */
