#ifndef __P3PPREFERENCES_H__
#define __P3PPREFERENCES_H__

#include "GContainer.h"
#include "GString.h"
#include "XmlFile.h"
#include "XmlNode.h"
#include "GURL.h"


class P3PPreferences:public GPEnabled
{
private:
	GList<GString> atts_empty, atts_reqd, atts_appel_connective;
	GList<GString> vals_empty, vals_always, vals_optout, vals_or;
	bool recipient_thirdparty;
	bool purpose_pseudo_analysis, purpose_pseudo_decision;
	bool purpose_individual_analysis, purpose_individual_decision;
	bool purpose_contact_always, purpose_contact_optout;
	bool purpose_telemarketing_always, purpose_telemarketing_optout;
	bool categories_personal, categories_medical, categories_financial;

	static GCriticalSection class_lock;

public:
	// Privacy Settings
	enum PrivacyLevels{LOW=1, MEDIUM, HIGH, CUSTOM_PRIVACY, IMPORTED_PRIVACY};
	PrivacyLevels privacy_level;

	bool warn_medical_analysis;
	bool warn_medical_share;

	bool warn_financial_analysis;
	bool warn_financial_share;
	
	bool warn_contact_mail;
	bool warn_contact_phone;
	bool warn_no_remove_mailinglist;
	bool warn_personal_analysis;
	bool warn_personal_share;
	bool warn_no_access_data;

	bool warn_nonpersonal_analysis;
	bool warn_nonpersonal_share;


	// Cookie Settings
	enum PrefValues{MATCHING_PREFERENCES, ALL, NONE, CUSTOM_COOKIE};
	PrefValues allow_cookies, allow_cookies_custom;
	PrefValues allow_cookies_custom_lastpagevisited, allow_cookies_custom_crosssite;

	// Other Settings
	bool check_upgrade;
	enum UserPrefs{DISABLE_ANIMATION=1, DISABLE_POPUP_WINDOWS=2};
	int user_prefs;
	enum SoundValues{DISABLE_SOUNDS=0, ENABLE_SOUND_UNKNOWN=1, ENABLE_SOUND_UNMATCHED=2, ENABLE_SOUND_MATCHED=4, PLAY_SOUND_CACHED_POLICY=8};
	int enable_sounds;

	enum HotKeyModValues{CTRL=1, ALT=2, SHIFT=4};  //binary so that bit-wise operations can be done
	int hotkey_mod;
	int hotkey_key;

	// Site Settings
	enum site_allow_cookies {ALWAYS, THIS_SESSION, NEVER};
	GMap<GString, site_allow_cookies> domain_prefs;

	GMap<GURL, bool> no_prompt_sites_list;
	bool dontwarn_forms;

	P3PPreferences()
	{
		// commonly used attributes and values
		atts_reqd.append("required");
		vals_always.append("always");		
		vals_optout.append("opt-out");		
		atts_appel_connective.append("appel:connective");
		vals_or.append("or");

	}

	~P3PPreferences()
	{
	}


public:

	int Load();
	int InitializationLoad(const char* lpszPathName);
	int CookieLoad(const char* lpszPathName);
	int PreferenceLoad(const char* lpszPathName);
	GString GetCurPreferenceFile(bool Reload);
	bool ReadPreferenceFile(DArray<char> &data);

	int Save();
	int InitializationSave(const char* lpszPathName);
	int CookieSave(const char* lpszPathName);
	int PreferenceSave(const char* lpszPathName);

	int NoPromptSitesLoad(const char* lpszPathName);
	int NoPromptSitesSave(const char* lpszPathName);
	GList<GURL> GetNoPromptSiteList();
	int SetNoPromptSiteList(GList<GURL>); 
	int GetNoPromptSiteListSize();
	bool IsInNoPromptSiteList(GURL url);
	void AddToNoPromptSiteList(GURL url, bool flag);

	int SetNoPromptAllForms(bool flag); 
	bool GetNoPromptAllForms();


private:
	
	int ParseRuleElement(GP<XmlNode> node);
	int ParsePolicyElement(GP<XmlNode> node);
	int ParseAccessElement(GP<XmlNode> node);
	int ParseStatementElement(GP<XmlNode> node);
	int ParsePurposeElement(GP<XmlNode> node);
	int ParseRecipientElement(GP<XmlNode> node);
	int ParseDataGroupElement(GP<XmlNode> node);
	int ParseDataElement(GP<XmlNode> node);
	int ParseCategoriesElement(GP<XmlNode> node);

	int PersonalInfo(XmlOutFile &xmlf);
	int MedicalAnalysisAlways(XmlOutFile &xmlf);
	int MedicalShareAlways(XmlOutFile &xmlf);
	int FinancialAnalysisAlways(XmlOutFile &xmlf);
	int FinancialShareAlways(XmlOutFile &xmlf);
	int ContactMailAlways(XmlOutFile &xmlf);
	int ContactPhoneAlways(XmlOutFile &xmlf);
	int NoRemoveMailingList(XmlOutFile &xmlf);
	int PersonalAnalysisAlways(XmlOutFile &xmlf);
	int PersonalShareAlways(XmlOutFile &xmlf);
	int NoAccessData(XmlOutFile &xmlf);
	int NonPersonalAnalysisAlways(XmlOutFile &xmlf);
	int NonPersonalShareAlways(XmlOutFile &xmlf);
	int UnknownPurposeAlways(XmlOutFile &xmlf);
	int Otherwise(XmlOutFile &xmlf);

	int MedicalAnalysisOptout(XmlOutFile &xmlf);
	int MedicalShareOptout(XmlOutFile &xmlf);
	int FinancialAnalysisOptout(XmlOutFile &xmlf);
	int FinancialShareOptout(XmlOutFile &xmlf);
	int ContactMailOptout(XmlOutFile &xmlf);
	int ContactPhoneOptout(XmlOutFile &xmlf);
	int PersonalAnalysisOptout(XmlOutFile &xmlf);
	int PersonalShareOptout(XmlOutFile &xmlf);
	int NonPersonalAnalysisOptout(XmlOutFile &xmlf);
	int NonPersonalShareOptout(XmlOutFile &xmlf);
	int UnknownPurposeOptout(XmlOutFile &xmlf);

	int RecipientShare(XmlOutFile &xmlf, GList<GString> vals);
	int DataGroup(XmlOutFile &xmlf, bool medical, bool financial, bool personal_identification);
	int PurposeAnalysis(XmlOutFile &xmlf, GList<GString> vals, bool pseudo_analysis, bool pseudo_decision, bool individual_analysis, bool individual_decision, bool contact, bool telemarketing);

public:
		
	static GMap<GString, GMap<GString, bool> > cookie_history;
	void AddCookieHistory(const char *sitedomain, const char*trackerdomain);
	void SaveCookieHistory();
	void LoadCookieHistory();

};


extern P3PPreferences g_p3p_preferences;

#endif


/*
	 purposevalue=

 "<current" [required] "/>"             | ; Completion and Support of Current Activity
"<admin" [required]   "/>"             | ; Web Site and System Administration
"<develop" [required] "/>"             | ; Research and Development
"<customization" [required] "/>"       | ; Affirmative Customization
"<tailoring" [required] "/>"           | ; One-time Tailoring
"<pseudo-analysis" [required] "/>"     | ; Pseudonymous Analysis
"<pseudo-decision" [required] "/>"     | ; Pseudonymous Decision
"<individual-analysis" [required] "/>" | ; Individual Analysis
"<individual-decision" [required] "/>" | ; Individual Decision
"<contact" [required] "/>"             | ; Contacting Visitors for Marketing of Services or Products
"<historical" [required] "/>"          | ; Historical Preservation
"<telemarketing" [required] "/>"       | ; Telephone Marketing
"<other-purpose" [required] ">" PCDATA "</other-purpose>"; Other Uses

 
	category =

 "<physical/>"    | ; Physical Contact Information
"<online/>"      | ; Online Contact Information
"<uniqueid/>"    | ; Unique Identifiers
"<purchase/>"    | ; Purchase Information
"<financial/>"   | ; Financial Information
"<computer/>"    | ; Computer Information
"<navigation/>"  | ; Navigation and Click-stream Data
"<interactive/>" | ; Interactive Data
"<demographic/>" | ; Demographic and Socioeconomic Data
"<content/>"     | ; Content
"<state/>"       | ; State Management Mechanisms
"<political/>"   | ; Political Information
"<health/>"      | ; Health Information
"<preference/>"  | ; Preference Data
"<government/>   | ; Government-issued Identifiers
"<other-category>" PCDATA "</other-category>" ; Other
 
*/