
#include "stdafx.h"
#include "GString.h"
#include "SummaryHTML.h"
#include "stdio.h"
#include "globals.h"
#include "resource.h"

#define UNMATCHCOLOR	"#FFCCCC"
#define MATCHCOLOR		"#C0FFC0"

#define opt_uri_link "<a href=\"%s\" target=\"_blank\">%s</a>"


GString SummaryHTML::PrepareDLLPath(const GString &formatstring)
{
	GString buffer;
	static GString path;
	if( path.length() == 0 )
	{
		path = g_dllpath+"/";
		for(int i=0; i<(int)path.length(); i++)
			if( path[i] == '\\' )
				path.setat(i, '/');
	}
	buffer.format(formatstring, (const char*)path );

	return buffer;
}


GString SummaryHTML::MarkupOptURI(const GString &str, const GString &opturl)
{
	if( !opturl.length() || !str.length())
		return str;

	GString result;
	const char *data = str;
	while(*data)
	{
		const char *optpos = strstr(data, "#opt-out#");
		if( optpos )
		{
			GString formatbuffer;
			result += GString(data, optpos-data);
			formatbuffer.format(opt_uri_link, (const char*)opturl, "opt-out");
			result += formatbuffer;
			data = optpos + strlen("#opt-out#");
		}
		else
		{
			result += data;
			break;
		}
	}
	return result;
}



GString SummaryHTML::ReplaceStatementFormatString(const GString &poundname, const GString &formatstring, const GP<StatementInfo> statement, int statement_number)
{
	GString buffer;
	//make buffer to a null string
	//buffer[0] = 0;

	if( poundname == "Number" && statement_number>0)
	{
		buffer.format(formatstring, statement_number);		
	}
	else if( poundname == "DLLPath" )
	{
		return PrepareDLLPath(formatstring);
	}
	else if( poundname == "Name" && statement->ExtensionName.length() )
	{
		buffer.format( formatstring, (const char*)statement->ExtensionName);		
	}
	else if( poundname == "InfoCollectedHeading" && (! statement->InfoCollected.isempty() ) )
	{
		buffer.format( formatstring);		
	}
	else if( poundname == "InfoCollected" )
	{
		GString info;
		for(GPosition pos=statement->InfoCollected; pos; ++pos)
		{
			GString catinfo;
			if( statement->InfoCollected[pos].CatInfo.size()>0)
			{
				
				catinfo = "<ul>";
				for(GPosition catpos=statement->InfoCollected[pos].CatInfo; catpos; ++catpos)
				{
					GString temp;
					temp.format("<li>%s</li>", (const char*)statement->InfoCollected[pos].CatInfo[catpos]);
					catinfo += temp;
				}
				catinfo += "</ul>";
			}
			buffer.format( formatstring, (const char*)(statement->InfoCollected[pos].DataCollected+catinfo));
		
			info+=buffer;
		}
		return info;
	}
	else if( poundname == "PurposesHeading" && (! statement->Purposes.isempty() ) )
	{
		buffer.format( formatstring);		
	}
	else if( poundname == "Purposes" )
	{
		GString purp;
		for(GPosition pos=statement->Purposes; pos; ++pos)
		{
			buffer.format( formatstring, (const char*)statement->Purposes[pos]);
			purp+=buffer;
		}
		return purp;
	}
	else if( poundname == "RecipientsHeading" && (! statement->Recipients.isempty() ) )
	{
		buffer.format( formatstring);		
	}
	else if( poundname == "Recipients" )
	{
		GString recp;
		for(GPosition pos=statement->Recipients; pos; ++pos)
		{
			buffer.format( formatstring, (const char*)statement->Recipients[pos]);
			recp+=buffer;
		}
		return recp;
	}
	else if( poundname == "Consequence" && statement->Consequence.length() && !IsAllWhiteSpace(statement->Consequence) )
	{
		buffer.format( formatstring, (const char*)statement->Consequence);
	}

	return buffer;
}



void SummaryHTML::PrepareStatementHTML(GP<StatementInfo> result_statement, GString &resulthtml, int statement_number)
{

	HRSRC rsrc = FindResource(hinst, MAKEINTRESOURCE(IDR_STATEMENTTEMPLATE), RT_HTML );

	if( !rsrc )
		return;

	HGLOBAL p = LoadResource(hinst, rsrc);
	if( !p )
		return;

	char *d = (char*)LockResource(p);
	int size = SizeofResource(hinst, rsrc);

	if( !d )
		return;

	GString htmldata(d, size);

	const char *data = htmldata;

	const char *datapos = data;
	while(*datapos)
	{
		char *firstpoundpos = strstr(datapos, "##");
		if( !firstpoundpos )
		{
			resulthtml += datapos;
			break;
		}

		char *secondpoundpos = strstr(firstpoundpos+2, "##");
		if( !secondpoundpos )
		{
			resulthtml += datapos;
			break;
		}

		char *fatpos = secondpoundpos+2;
		if( *fatpos != '@' )
		{
			resulthtml += GString(datapos, secondpoundpos-datapos+2);
			datapos = secondpoundpos+2;
			continue;
		}

		char *satpos = strchr(fatpos+1, '@');
		if( !satpos )
		{
			resulthtml += datapos;
			break;
		}

		GString poundname(firstpoundpos+2, secondpoundpos-firstpoundpos-2);
		GString formatstring(fatpos+1, satpos-fatpos-1);

		resulthtml += GString(datapos, firstpoundpos-datapos);
		datapos = satpos+1;

		resulthtml += ReplaceStatementFormatString(poundname, formatstring, result_statement, statement_number);

	}
	return;
}


GString SummaryHTML::ReplaceFormatString(const GString &poundname, const GString &formatstring, const Result &result)
{
	GString buffer;
	//make buffer to a null string
	//buffer[0] = 0;

	if( poundname == "DLLPath" )
	{
		return PrepareDLLPath(formatstring);
	}
	else
	if( poundname == "BusinessName" && result.BusinessName.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessName);
	}
	else if( poundname == "BusinessPostalName" && result.BusinessPostalName.length())
	{
		buffer.format( formatstring, result.BusinessPostalName);
	}
	else if( poundname == "BusinessPostalStreet" && result.BusinessPostalStreet.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessPostalStreet);
	}
	else if( poundname == "BusinessPostalCity" && result.BusinessPostalCity.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessPostalCity);
	}
	else if( poundname == "BusinessPostalStateprov" && result.BusinessPostalState.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessPostalState);
	}
	else if( poundname == "BusinessPostalCode" && result.BusinessPostalCode.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessPostalCode);
	}
	else if( poundname == "BusinessPostalCountry" && result.BusinessPostalCountry.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessPostalCountry);
	}
	else if( poundname == "PhoneHeading" && 
			( result.BusinessTelephoneIntCode.length() || result.BusinessTelephoneLocCode.length() 
			 || result.BusinessTelephoneNumber.length() || result.BusinessTelephoneExt.length()
			 || result.BusinessTelephoneComment.length() ) )
	{
		buffer.format( formatstring);
	}
	else if( poundname == "BusinessTelephoneIntCode" && result.BusinessTelephoneIntCode.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessTelephoneIntCode);
	}
	else if( poundname == "BusinessTelephoneLocCode" && result.BusinessTelephoneLocCode.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessTelephoneLocCode);
	}
	else if( poundname == "BusinessTelephoneNumber" && result.BusinessTelephoneNumber.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessTelephoneNumber);
	}
	else if( poundname == "BusinessTelephoneExt" && result.BusinessTelephoneExt.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessTelephoneExt);
	}
	else if( poundname == "BusinessTelephoneComment" && result.BusinessTelephoneComment.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessTelephoneComment);
	}
	else if( poundname == "FaxHeading" && 
			( result.BusinessFaxIntCode.length() || result.BusinessFaxLocCode.length() 
			 || result.BusinessFaxNumber.length() || result.BusinessFaxExt.length()
			 || result.BusinessFaxComment.length() ) )
	{
		buffer.format( formatstring);
	}
	else if( poundname == "BusinessFaxIntCode" && result.BusinessFaxIntCode.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessFaxIntCode);
	}
	else if( poundname == "BusinessFaxLocCode" && result.BusinessFaxLocCode.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessFaxLocCode);
	}
	else if( poundname == "BusinessFaxNumber" && result.BusinessFaxNumber.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessFaxNumber);
	}
	else if( poundname == "BusinessFaxExt" && result.BusinessFaxExt.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessFaxExt);
	}
	else if( poundname == "BusinessFaxComment" && result.BusinessFaxComment.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessFaxComment);
	}
	else if( poundname == "BusinessOnlineEmail" && result.BusinessOnlineEmail.length())
	{
		buffer.format( formatstring, (const char*)result.BusinessOnlineEmail, (const char*)result.BusinessOnlineEmail);
	}
	else if( poundname == "BusinessOnlineURI" && result.BusinessOnlineURI.length())
	{
		const char* uri = (const char*)result.BusinessOnlineURI;
		buffer.format( formatstring, uri, uri, uri);
	}
	else if( (poundname == "DisputesHeading") && (! result.Disputes.isempty() ) )
	{
			buffer.format( formatstring);
	}
	else if( (poundname == "DisputesHeadingExpandGroup") && (! result.Disputes.isempty() ) )
	{
			return PrepareDLLPath(formatstring);
	}
	else if( poundname == "Disputes" )
	{
		GString disp;
		int i = 1;
		for(GPosition pos=result.Disputes; pos; ++pos, i++)
		{
			GP<DisputeInfo> di = result.Disputes[pos];
/*			GString description;
			if( di->ShortDescription.length() )
			{
				description = di->ShortDescription;
			}
			else if( di->LongDescription.length() )
			{
				description = di->LongDescription;
			}
			else
				description = "";
*/			
			
			GString image="";
			if( di->ImageURL.length() )
			{
				image="<img align=\"left\" hspace=5 src=\"" + di->ImageURL + "\" alt=\"" + di->ImgAltText + "\">";
			}
			GString longdesc="";
			if( di->LongDescription.length() )
			{
				longdesc=": " + di->LongDescription;
			}
			buffer.format( formatstring, (const char*)image, (const char*)di->ServiceURL, i, (const char*)di->ShortDescription, (const char*)di->ShortDescription, (const char*)longdesc);
			disp+=buffer;

		}
		return disp;
	}
	else if( poundname == "MatchNoMatch" )
	{
		if( result.MisMatches.size() )
		{
			buffer.format( formatstring, "does not match your preferences:");
		}
		else
		{
			buffer.format( formatstring, "matches your preferences.");
		}
	}
	else if( poundname == "MatchNoMatchExpandGroup" )
	{
		if( result.MisMatches.size() )
		{
			return PrepareDLLPath(formatstring);
		}
	}
	else if( poundname == "BGColor" )
	{
		if( result.MisMatches.size() )
		{
			buffer.format( formatstring, UNMATCHCOLOR);
		}
		else
		{
			buffer.format( formatstring, MATCHCOLOR);
		}
	}
	else if( poundname == "MisMatches" )
	{
		GString mises;
		for(GPosition pos=result.MisMatches; pos; ++pos)
		{
			buffer.format( formatstring, (const char*)result.MisMatches[pos]);
			mises+=buffer;
		}
		return mises;
	}
	else if( poundname == "StatementList" )
	{
		int i = 1;
		GString st_list;
		for(GPosition pos=result.Statements; pos; ++pos, i++)
		{
			if( result.Statements[pos]->ExtensionName.length() )
			{
				const char* stmt_name = (const char*)result.Statements[pos]->ExtensionName;
				buffer.format( formatstring, i, i, "-", stmt_name, i, "-", stmt_name);
			}
			else
			{
				buffer.format( formatstring, i, i, "", "", i, "", "");
			}
			st_list+=buffer;
		}
		return st_list;
	}
	else if( poundname == "Statements" )
	{
		GString stmts;
		int j= 1;
		for(GPosition pos=result.Statements; pos; ++pos, ++j)
		{
			GP<StatementInfo> st = result.Statements[pos];
			GString stmthtml;
			PrepareStatementHTML(st, stmthtml, result.Statements.size()>1?j:0);
			stmts += stmthtml;
		}
		return stmts;
	}
	else if( poundname == "AccessHeading" && result.Access.length())
	{
		buffer.format( formatstring);		
	}
	else if( poundname == "AccessHeadingExpandGroup" && result.Access.length())
	{
		return PrepareDLLPath(formatstring);		
	}
	else if( poundname == "Access" && result.Access.length())
	{
		buffer.format( formatstring, (const char*)result.Access);		
	}
	else if( poundname == "FullPolicyURI" && result.DiscURI.length())
	{
		GURL DiscURI(result.DiscURI, result.appliedPolicyURL);
		buffer.format( formatstring, (const char*)DiscURI);
	}
	else if( poundname == "OPTURI" && result.OptURI.length())
	{
		GURL OptURI(result.OptURI, result.appliedPolicyURL);
		buffer.format( formatstring, (const char*)OptURI);
	}
	else if( poundname == "Test" && result.Test )
	{
		buffer.format( formatstring);
	}

	return buffer;
}

void SummaryHTML::PrepareSummary(Result &result, GString &formatResource, GString &resulthtml)
{
	const char *data = formatResource;

	const char *datapos = data;
	while(*datapos)
	{
		char *firstpoundpos = strstr(datapos, "##");
		if( !firstpoundpos )
		{
			resulthtml += datapos;
			break;
		}

		char *secondpoundpos = strstr(firstpoundpos+2, "##");
		if( !secondpoundpos )
		{
			resulthtml += datapos;
			break;
		}

		char *fatpos = secondpoundpos+2;
		if( *fatpos != '@' )
		{
			resulthtml += GString(datapos, secondpoundpos-datapos+2);
			datapos = secondpoundpos+2;
			continue;
		}

		char *satpos = strchr(fatpos+1, '@');
		if( !satpos )
		{
			resulthtml += datapos;
			break;
		}

		GString poundname(firstpoundpos+2, secondpoundpos-firstpoundpos-2);
		GString formatstring(fatpos+1, satpos-fatpos-1);

		resulthtml += GString(datapos, firstpoundpos-datapos);
		datapos = satpos+1;

		resulthtml += ReplaceFormatString(poundname, formatstring, result);

	}
}


void SummaryHTML::PrepareSummaryHTML(Result &result, GString &resulthtml)
{
	resulthtml.empty();

	HRSRC rsrc = FindResource(hinst, MAKEINTRESOURCE(IDR_SUMMARYTEMPLATE), RT_HTML );

	if( !rsrc )
		return;

	HGLOBAL p = LoadResource(hinst, rsrc);
	if( !p )
		return;

	char *d = (char*)LockResource(p);
	int size = SizeofResource(hinst, rsrc);

	if( !d )
		return;

	GString htmldata(d, size);
	
	PrepareSummary(result, htmldata, resulthtml);
	
	if( result.OptURI.length() )
	{
		GURL optURI(result.OptURI, result.appliedPolicyURL);
		resulthtml = MarkupOptURI(resulthtml, GString(optURI));
	}
	return;
}


void SummaryHTML::PrepareShortSummary(Result &result, GString &resultSummary)
{
	GString htmldata;//("##BusinessName##@%s's@ privacy policy  ##MatchNoMatch##@%s@ \n##MisMatches##@\n%s@");
	switch( result.Match )
	{
	case Result::MATCH:
		htmldata = "##BusinessName##@%s's@ privacy policy  ##MatchNoMatch##@%s@";
		break;
	case Result::UNMATCH:
		htmldata = "##BusinessName##@%s's@ privacy policy  ##MatchNoMatch##@%s@ \n##MisMatches##@\n%s@";
		break;
	default:
		htmldata = "Valid policy not found";
	}

	PrepareSummary(result, htmldata, resultSummary);
}