// URLListDialog.cpp : implementation file
//

#include "stdafx.h"

#include "commctrl.h"
#include "URLListDialog.h"
#include "P3PPreferences.h"
#include "globals.h"



/////////////////////////////////////////////////////////////////////////////
// CURLListDialog dialog


CURLListDialog::CURLListDialog(GList<GURL> &u_list): url_list(u_list)
{
}

CURLListDialog::CURLListDialog(GList<GURL> &u_list, GString title, GString text): url_list(u_list), window_title(title), static_text(text)
{
}

CURLListDialog::~CURLListDialog()
{
}


LRESULT CURLListDialog::OnInitDialog(UINT uMsg, WPARAM wParam, 
	LPARAM lParam, BOOL& bHandled)
{
	SetWindowText(window_title);	
//	HWND texthwnd=GetDlgItem(IDC_STATIC_LISTHEADING);
	SetDlgItemText(IDC_STATIC_LISTHEADING, static_text);	

	EnableButtons(false);

	switch (g_p3p_preferences.GetNoPromptAllForms())
	{
		case true :
			CheckDlgButton(IDC_NOPROMPT_ALLFORMS, BST_CHECKED); 
			::EnableWindow(GetDlgItem(IDC_URL_LIST), FALSE);
			break;

		case false :
			CheckDlgButton(IDC_NOPROMPT_ALLFORMS, BST_UNCHECKED); 
			::EnableWindow(GetDlgItem(IDC_URL_LIST), TRUE);
			break;
	}

	HWND listhwnd=GetDlgItem(IDC_URL_LIST);
	if( listhwnd )
	{
		ListView_SetExtendedListViewStyleEx(listhwnd, LVS_SHOWSELALWAYS, 0);
		
		LVITEM lvi;
				
		lvi.mask = LVIF_TEXT | LVIF_PARAM | LVIF_STATE;
		lvi.state = 0;     
		lvi.stateMask = 0; 
		
		int index = 0;
		for( GPosition pos = url_list; pos; ++pos, index++ )
		{
			lvi.iItem = index;
			lvi.iSubItem = 0; 
			GURL url = url_list[pos];
			lvi.pszText = (char*)(const char *)url;
			int i = ListView_InsertItem(listhwnd, &lvi);

		}
	}

	return 1;  
}


/////////////////////////////////////////////////////////////////////////////
// OnDestroy - Responds to the WM_DESTROY message
// 
LRESULT CURLListDialog::OnDestroy(UINT, WPARAM, LPARAM, BOOL&)
{

	return 1;  
}


LRESULT CURLListDialog::OnOK(WORD, WORD, HWND, BOOL&)
{
	bool dontwarn_allsites = IsDlgButtonChecked(IDC_NOPROMPT_ALLFORMS)==BST_CHECKED?true:false;
	if( g_p3p_preferences.GetNoPromptAllForms() != dontwarn_allsites )
	{
		g_p3p_preferences.SetNoPromptAllForms(dontwarn_allsites);
	}
	
	HWND ListWnd=GetDlgItem(IDC_URL_LIST);
	int count=ListView_GetItemCount(ListWnd);
	if( count < url_list.size() )  
	{
		////some are deleted
		url_list.empty();

		char buffer[2048];
		LVITEM lvi;
					
		lvi.mask = LVIF_TEXT | LVIF_PARAM | LVIF_STATE;
		lvi.state = 0;     lvi.stateMask = 0; 
		lvi.lParam=0;

		lvi.iSubItem = 0;
		lvi.pszText = buffer;
		lvi.cchTextMax = sizeof(buffer);

		for(int i=0; i<count; i++)
		{
			lvi.iItem = i;  // index of the item to retrieve

			if( ListView_GetItem(ListWnd, &lvi) )
			{
				GURL url = buffer; 
				url_list.append(url);
			}
		}
	}

	EndDialog(IDOK);
	return 1;

}

LRESULT CURLListDialog::OnContextHelp(UINT id, WPARAM wParam, LPARAM lParam, BOOL& bFlag)
{
	::MessageBeep(MB_OK);
	return 1;
}

LRESULT CURLListDialog::OnNotify(UINT code, WPARAM wParam, LPARAM lParam, BOOL& result)
{
	LPNMHDR pmhdr = (LPNMHDR) lParam;
	if( pmhdr->idFrom == IDC_URL_LIST )
	{
		unsigned int i = ListView_GetSelectedCount(GetDlgItem(IDC_URL_LIST));
		EnableButtons(i);
	}
	return FALSE;
}


LRESULT CURLListDialog::OnDelete(WORD, WORD, HWND, BOOL&)
{
	HWND ListWnd=GetDlgItem(IDC_URL_LIST);
	int i=ListView_GetNextItem(ListWnd, -1, LVNI_SELECTED);
	if( i >= 0 )
	{
		char buffer[2048];
		LVITEM lvi;
				
		lvi.mask = LVIF_TEXT | LVIF_PARAM | LVIF_STATE;
		lvi.state = 0;     lvi.stateMask = 0; 
		lvi.lParam=0;
		lvi.iItem = i;
		lvi.iSubItem = 0;
		lvi.pszText = buffer;
		lvi.cchTextMax = sizeof(buffer);

		if( ListView_GetItem(ListWnd, &lvi) )
		{
			return( ListView_DeleteItem(ListWnd, i) );
		}
	}

return FALSE;
}


LRESULT CURLListDialog::OnCancel(WORD, WORD, HWND, BOOL&)
{
	EndDialog(IDCANCEL);
	return 1;
}


void CURLListDialog::EnableButtons(int enable)
{
	if( enable )
	{
		::EnableWindow(GetDlgItem(ID_DELETE), TRUE);
	}
	else
	{
		::EnableWindow(GetDlgItem(ID_DELETE), FALSE);
	}
}

int CURLListDialog::GetSelectedItemLParam()
{
	HWND ListWnd=GetDlgItem(IDC_URL_LIST);
	int i=ListView_GetNextItem(ListWnd, -1, LVNI_SELECTED);
	if( i >= 0 )
	{
		LVITEM lvi;
				
		lvi.mask = LVIF_PARAM;
		lvi.state = 0;     
		lvi.stateMask = 0; 
		lvi.lParam=0;
		lvi.iItem = i;
		lvi.iSubItem = 0;
		if( ListView_GetItem(ListWnd, &lvi) )
		{
			return lvi.lParam;
		}
	}
	return -1;
}

LRESULT CURLListDialog::OnNoPromptAllForms(WORD, WORD, HWND, BOOL&)
{
	if( IsDlgButtonChecked(IDC_NOPROMPT_ALLFORMS) )
	{
		::EnableWindow(GetDlgItem(IDC_URL_LIST), FALSE);
		ListView_SetItemState(GetDlgItem(IDC_URL_LIST), -1, !LVIS_SELECTED, LVIS_SELECTED);
	}
	else
	{
		::EnableWindow(GetDlgItem(IDC_URL_LIST), TRUE);
		ListView_SetItemState(GetDlgItem(IDC_URL_LIST), -1, !LVIS_SELECTED, LVIS_SELECTED);
	}

	return 1;
}


