#include "XmlNode.h"

#ifndef NO_SAX

#include "SAXElemHandler.h"
#include <util/PlatformUtils.hpp>
#include <util/TransService.hpp>
#include <parsers/SAXParser.hpp>
#include <framework/MemBufInputSource.hpp>

static const char*              encodingName    = "LATIN1";
static XMLFormatter::UnRepFlags unRepFlags      = XMLFormatter::UnRep_CharRef;
static const char*  gMemBufId = "P3P";

#endif


GP<XmlNode> SaxXmlParse(const char *buffer, int len, bool schemacheck) {
  // Initialize the XML4C2 system
#ifdef NO_SAX
  return XMLParse(buffer, len);
#else
  try {
    XMLPlatformUtils::Initialize();
  }

  catch (const XMLException& toCatch) {
     cerr << "Error during initialization! :\n"
       << StrX(toCatch.getMessage()) << endl;
	    return NULL;
  }
 
  int errorCount = 0;

    //
    //  Create a SAX parser object. Then, according to what we were told on
    //  the command line, set it to validate or not.
    //
  SAXParser* parser = new SAXParser;
  //parser->setValidationScheme(valScheme);
  parser->setDoNamespaces(schemacheck);
  parser->setDoSchema(schemacheck);
  //parser->setValidationSchemaFullChecking(schemaFullChecking);
  parser->setExternalSchemaLocation(
				    "http://www.w3.org/2001/09/P3Pv1 C:\\p3pv1.xml "
				    "http://www.w3.org/2000/12/P3Pv1 C:\\schema.xsd "
				    );

  //
  //  Create the handler object and install it as the document and error
  //  handler for the parser-> Then parse the file and catch any exceptions
  //  that propogate out
  //
  SAXElemHandlers handler(encodingName, unRepFlags);
  try {
    MemBufInputSource* memBufIS = new MemBufInputSource
      (
       (const XMLByte*)buffer
       , len
       , gMemBufId
       , false
       );
			
    parser->setDocumentHandler(&handler);
    parser->setErrorHandler(&handler);
    parser->parse(*memBufIS);
    errorCount = parser->getErrorCount();
  }

  catch (const XMLException& toCatch) {
    /*
      cerr << "\nAn error occured\n  Error: "
      << StrX(toCatch.getMessage())
      << "\n" << endl;
			 */
    delete parser;
    XMLPlatformUtils::Terminate();

		//THROW(StrX(toCatch.getMessage());
    return NULL;
  }

  //
  //  Delete the parser itself.  Must be done prior to calling Terminate, below.
  //
  delete parser;

  // And call the termination method
  XMLPlatformUtils::Terminate();

  if (errorCount > 0)
    return NULL;
  else
    return handler.root;
#endif
}



GP<XmlNode> SaxXmlParse(const char*filename, bool schemacheck) {
#ifdef NO_SAX
  return XMLParse(filename);
#else
  // Initialize the XML4C2 system
  try {
    XMLPlatformUtils::Initialize();
  }

  catch (const XMLException& toCatch) {
    /* cerr << "Error during initialization! :\n"
       << StrX(toCatch.getMessage()) << endl;
       */
    return NULL;
  }
 
  int errorCount = 0;

  //
  //  Create a SAX parser object. Then, according to what we were told on
  //  the command line, set it to validate or not.
  //
  SAXParser* parser = new SAXParser;
  //parser->setValidationScheme(valScheme);
  parser->setDoNamespaces(schemacheck);
  parser->setDoSchema(schemacheck);
  //parser->setValidationSchemaFullChecking(schemaFullChecking);
  parser->setExternalSchemaLocation(
				    "http://www.w3.org/2001/09/P3Pv1 C:\\p3pv1.xml "
				    "http://www.w3.org/2000/12/P3Pv1 C:\\schema.xsd "
				    );

  //
  //  Create the handler object and install it as the document and error
  //  handler for the parser-> Then parse the file and catch any exceptions
  //  that propogate out
  //
  SAXElemHandlers handler(encodingName, unRepFlags);
  try {
        
    parser->setDocumentHandler(&handler);
    parser->setErrorHandler(&handler);
    parser->parse(filename);
    errorCount = parser->getErrorCount();
  }

  catch (const XMLException& toCatch) {
    /*
      cerr << "\nAn error occured\n  Error: "
      << StrX(toCatch.getMessage())
      << "\n" << endl;
      */
    delete parser;
    XMLPlatformUtils::Terminate();

    //THROW(StrX(toCatch.getMessage());
    return NULL;
  }

  //
  //  Delete the parser itself.  Must be done prior to calling Terminate, below.
  //
  delete parser;

  // And call the termination method
  XMLPlatformUtils::Terminate();

  if (errorCount > 0)
    return NULL;
  else
    return handler.root;
#endif
}
