#include <stdafx.h>
#include <wininet.h>
#include <string.h>
#include <stdio.h>
#include "GThreads.h"
#include "P3PClient.h"
#include "P3PObject.h"
#include "ReadHTTPFile.h"


BOOL Succeeded(HANDLE h, LPCTSTR strFunctionName)
{
   if (h == NULL)
   {
      DWORD dwError = GetLastError() ;

      GString errString;
      errString.format("%s returns error %u", strFunctionName, dwError) ;
      //MessageBox(NULL, errString, "Error", MB_ICONEXCLAMATION | MB_OK) ;
      return FALSE;
   }
   else
   {
      return TRUE ;
   }

}
UINT GetHTTPFileContent(LPCTSTR server, unsigned short port, bool securehttp, LPCTSTR path, char *&data, GString &header, bool headeronly=false);
void GetHTTPFileContentAsync(LPCTSTR server, unsigned short port, bool securehttp, LPCTSTR path, char *&data, GString &p3pheader, bool headeronly);

bool GetP3PHeaderFromCache(const char* rAddress, GString &p3pheader)
{
	unsigned long buffsize=0;
	int found = GetUrlCacheEntryInfoEx(rAddress, NULL, &buffsize, NULL, NULL, NULL, 0);
	if( buffsize > 0 )
	{
		GString buffer;
		LPINTERNET_CACHE_ENTRY_INFO intbuffer = (LPINTERNET_CACHE_ENTRY_INFO)buffer.getbuf(buffsize+1);
		found = GetUrlCacheEntryInfoEx(rAddress, intbuffer, &buffsize, NULL, NULL, NULL, 0);
		if( found )
		{
			GString intheader = (char*)intbuffer->lpHeaderInfo;
			int i = intheader.search("\nP3P:");
			if( i >= 0 )
			{
				i+=5;
				int j = intheader.search("\n", i);
				if( j >= 0 )
				{
					p3pheader = intheader.substr(i, j-i);
					return true;
				}
			}
		}
	}
	return false;
}


char* GetPage(const char* rAddress, GString &p3pheader, bool headeronly)
{
   //
   // Parse the http address for the server and file names
   //
	if( headeronly )
	{
		if( GetP3PHeaderFromCache(rAddress, p3pheader) )
			return NULL;		
	}

   int len = strlen(rAddress);
   if( len )
   {
	   bool securehttp = false;

	   GString str;
	   char *buffer = str.getbuf(len+1);
	   strcpy(buffer, rAddress);

	   char *server = 0;
	   char *path = 0;
	   unsigned short port = 0;

	   if( strncmp(buffer, "http://", 7) == 0 )
	   {
			server = &buffer[7]; // Remove http://
			securehttp = false;
	   }
	   else if( strncmp(buffer, "https://", 8) == 0 )
	   {
			server = &buffer[8]; // Remove https://
			securehttp = true;
	   }
	   else 
	   {
		   return NULL;
	   }

	   char *c = strchr(server, '/');//(strTemp.Find("/"); // Filename on address?
	   if ( c )
	   {
		   *c = 0;
		   path = c+1;
	   }
	   c = strchr(server, ':'); /// find the port number if used
	   if( c )
	   {
		   *c = 0;
		   port = atoi(c+1);
	   }


	   char *data=NULL;
	   //GetHTTPFileContent(server, port, securehttp, path, data, p3pheader, headeronly);
	   GetHTTPFileContentAsync(server, port, securehttp, path, data, p3pheader, headeronly);

	   return data;
   }
   return NULL;
   
}

struct DownloadInfo
{
	DownloadInfo(LPCTSTR _server, unsigned short _port, bool _securehttp, LPCTSTR _path, 
		char *&_data, GString &_p3pheader, bool _headeronly):
		server(_server), port(_port), securehttp(_securehttp), path(_path), 
		 data(_data), p3pheader(_p3pheader), headeronly(_headeronly), 
		 hConnect(0), hHttpFile(0), monitor(0) {}
	LPCTSTR server;
	unsigned short port;
	bool securehttp;
	LPCTSTR path;
	char *&data;
	GString &p3pheader;
	bool headeronly;
	HINTERNET hConnect;
	HINTERNET hHttpFile;
	GMonitor *monitor;
	GCriticalSection section;

};
void GetHTTPFileContentInThread(void *info);

void GetHTTPFileContentAsync(LPCTSTR server, unsigned short port, bool securehttp, LPCTSTR path, char *&data, GString &p3pheader, bool headeronly)
{

	DownloadInfo dinfo(server, port, securehttp, path, data, p3pheader, headeronly);
	{
		
		dinfo.monitor = CP3PObject::GetWorkerThreadMonitor(GetCurrentThreadId());
			
		GThread downloadthread;

		downloadthread.create(GetHTTPFileContentInThread, &dinfo);
		
		if( dinfo.monitor )
		{
			dinfo.monitor->enter();
			dinfo.monitor->wait(5*60*1000);
			dinfo.monitor->leave();
		}


	
		{
			GCriticalSectionLock lock(&dinfo.section);
			bool throw_exep = false;
			if( dinfo.hHttpFile )
			{
				InternetCloseHandle(dinfo.hHttpFile);
				dinfo.hHttpFile = 0;
				throw_exep = true;
			}
			if( dinfo.hConnect )
			{
				InternetCloseHandle(dinfo.hConnect);
				dinfo.hConnect = 0;
				throw_exep = true;
			}
			if( throw_exep )
			{
				if( dinfo.data )
				{
					delete dinfo.data;
					dinfo.data = 0;
				}
				throw("STOP");
			}
		}
	}
}

void __stdcall MyInternetStatusCallback(HINTERNET hInternet,
    DWORD dwContext,
    DWORD dwInternetStatus,
    LPVOID lpvStatusInformation,
    DWORD dwStatusInformationLength)
{
	if( dwInternetStatus = INTERNET_STATUS_REQUEST_COMPLETE )
	{
		if( lpvStatusInformation )
		{
			LPINTERNET_ASYNC_RESULT lpv = (LPINTERNET_ASYNC_RESULT)lpvStatusInformation;
			//*((int*)dwContext) = 1;
		}
	}
	
}


UINT GetHTTPFileContent(LPCTSTR server, unsigned short port, bool securehttp, LPCTSTR path, char *&data, GString &p3pheader, bool headeronly)
{
	p3pheader.empty();
	static HINTERNET m_hSession=NULL ;

    UINT uiResult = -1;

    if (m_hSession == NULL)
	{
		//
		// Initialize the Internet Functions.
		//

		if( !m_hSession )
			m_hSession = ::InternetOpen("P3P Client",
									  PRE_CONFIG_INTERNET_ACCESS,
									  NULL,
									  NULL,
									  0/*INTERNET_FLAG_NO_COOKIES /*INTERNET_FLAG_ASYNC*/) ;

		if (!Succeeded(m_hSession, "InternetOpen"))
		{
		 // Send message to UI that we finished.
		 return -1;
		}
	}
	
	volatile int zzz=0;
	
	HINTERNET hConnect = ::InternetConnect(m_hSession,
										  server,
										  port,
										  "",
										  "",
										  INTERNET_SERVICE_HTTP,
										  securehttp?INTERNET_FLAG_SECURE:0,
										  (DWORD)&zzz) ;

                                          
	if (Succeeded(hConnect, "InternetConnect"))
	{
		

		HINTERNET hHttpFile = ::HttpOpenRequest(hConnect,
											  //headeronly?"HEAD":"GET",
											  "GET",
											  path,
											  HTTP_VERSION,
											  NULL,
											  0,
											  securehttp?
											  INTERNET_FLAG_SECURE | INTERNET_FLAG_IGNORE_CERT_CN_INVALID 
											  :0, //INTERNET_FLAG_DONT_CACHE,
											  (DWORD)&zzz) ;

		if (Succeeded(hHttpFile, "HttpOpenRequest"))
		{
			
			//INTERNET_STATUS_CALLBACK callback = ::InternetSetStatusCallback(hHttpFile, (INTERNET_STATUS_CALLBACK)MyInternetStatusCallback);

			BOOL bSendRequest = ::HttpSendRequest(hHttpFile, NULL, 0, 0, 0);

			
			/*BOOL bSendRequest = ::HttpSendRequestEx(hHttpFile, NULL, NULL, HSR_SYNC, (DWORD)&zzz);

			while(!zzz)
			{
				::Sleep(10);
			}
			*/
			if (Succeeded((HINTERNET)bSendRequest, "HttpSendRequest"))
			{
				// Get the header info
				{
					char *h = "P3P:";

					DWORD dwLengthBufQuery = strlen(h);
					::HttpQueryInfo(hHttpFile,
						   HTTP_QUERY_CUSTOM , 
						   h, 
						   &dwLengthBufQuery,
						   NULL) ;

					if( dwLengthBufQuery > strlen(h) )
					{
 
						char *hd=p3pheader.getbuf(dwLengthBufQuery+1);
						strcpy(hd, "P3P:");
						dwLengthBufQuery++;

						BOOL bQuery = ::HttpQueryInfo(hHttpFile,
						   HTTP_QUERY_CUSTOM , 
						   hd, 
						   &dwLengthBufQuery,
						   NULL) ;

					}
				}
				if( !headeronly )
				{
					// Get size of file.
					char bufQuery[32] ;
					DWORD dwFileSize ;
					DWORD dwLengthBufQuery = sizeof (bufQuery);
					BOOL bQuery = ::HttpQueryInfo(hHttpFile,
						   HTTP_QUERY_CONTENT_LENGTH, 
						   bufQuery, 
						   &dwLengthBufQuery,
						   NULL) ;

					if (Succeeded((HINTERNET)bQuery, "HttpQueryInfo"))
					{
						// The Query was successful, so allocate the memory.
						dwFileSize = (DWORD)atol(bufQuery) ;
					}
					else
					{
						// The Query failed. Allocate some memory. Should allocate memory in blocks.
						dwFileSize = 5*1024 ;
					}


					data = new char[dwFileSize+1024] ;
					DWORD dwBytesRead ;
					BOOL bRead = ::InternetReadFile(hHttpFile,	
											data,	
											dwFileSize+1, 	
											&dwBytesRead);	
					data[dwFileSize] = 0;
					if (Succeeded((HINTERNET)bRead, "InternetReadFile"))
					{
						data[dwBytesRead] = 0 ;
						uiResult = 0;
					} // InternetReadFile
				}
			} // HttpSendRequest

			::InternetCloseHandle(hHttpFile); 
		} // HttpOpenRequest

		::InternetCloseHandle(hConnect) ;
	} // InternetConnect 
   return uiResult ;
}



void GetHTTPFileContentInThread(void *info)
{
		DownloadInfo *dinfo = (DownloadInfo*)info;
	
try
{
	dinfo->p3pheader.empty();
	static HINTERNET m_hSession=NULL ;

    if (m_hSession == NULL)
	{
		//
		// Initialize the Internet Functions.
		//

		/////right now we are not blocking the cookies but in future
		///  we may have to block the cookies
		m_hSession = ::InternetOpen("P3P Client",
									  PRE_CONFIG_INTERNET_ACCESS,
									  NULL,
									  NULL,
									  0/*INTERNET_FLAG_NO_COOKIES/*INTERNET_FLAG_ASYNC*/);


		if (!Succeeded(m_hSession, "InternetOpen"))
		{
		 // Send message to UI that we finished.
			goto end;
		}
	}
	
	
	dinfo->hConnect = ::InternetConnect(m_hSession,
										  dinfo->server,
										  dinfo->port,
										  "",
										  "",
										  INTERNET_SERVICE_HTTP,
										  dinfo->securehttp?INTERNET_FLAG_SECURE:0,
										  0) ;

                                          
	if (Succeeded(dinfo->hConnect, "InternetConnect"))
	{
		

		dinfo->hHttpFile = ::HttpOpenRequest(dinfo->hConnect,
											  //headeronly?"HEAD":"GET",
											  "GET",
											  dinfo->path,
											  HTTP_VERSION,
											  NULL,
											  0,
											  dinfo->securehttp?
											  INTERNET_FLAG_SECURE | INTERNET_FLAG_IGNORE_CERT_CN_INVALID | INTERNET_FLAG_NO_COOKIES
											  :0 | INTERNET_FLAG_NO_COOKIES, //INTERNET_FLAG_DONT_CACHE,
											  0) ;

		if (Succeeded(dinfo->hHttpFile, "HttpOpenRequest"))
		{
			
			
			BOOL bSendRequest = ::HttpSendRequest(dinfo->hHttpFile, NULL, 0, 0, 0);

			if (Succeeded((HINTERNET)bSendRequest, "HttpSendRequest"))
			{
				// Get the header info
				{
					char h[1024];

					DWORD dwLengthBufQuery = sizeof(h);

					::HttpQueryInfo(dinfo->hHttpFile,
						   HTTP_QUERY_STATUS_CODE  , 
						   h, 
						   &dwLengthBufQuery,
						   NULL) ;

					if( dwLengthBufQuery==0 || GString(h) == "200" )
					{

						strcpy(h, "P3P:");
						dwLengthBufQuery = strlen(h);

						::HttpQueryInfo(dinfo->hHttpFile,
							   HTTP_QUERY_CUSTOM , 
							   h, 
							   &dwLengthBufQuery,
							   NULL) ;

						

						if( dwLengthBufQuery > strlen(h) )
						{
 
							char *hd=dinfo->p3pheader.getbuf(dwLengthBufQuery+1);
							strcpy(hd, "P3P:");
							dwLengthBufQuery++;

							BOOL bQuery = ::HttpQueryInfo(dinfo->hHttpFile,
							   HTTP_QUERY_CUSTOM , 
							   hd, 
							   &dwLengthBufQuery,
							   NULL) ;

						}
					
						if( !dinfo->headeronly )
						{
							// Get size of file.
							char bufQuery[32] ;
							DWORD dwFileSize ;
							DWORD dwLengthBufQuery = sizeof (bufQuery);
							BOOL bQuery = ::HttpQueryInfo(dinfo->hHttpFile,
								   HTTP_QUERY_CONTENT_LENGTH, 
								   bufQuery, 
								   &dwLengthBufQuery,
								   NULL) ;

							if (Succeeded((HINTERNET)bQuery, "HttpQueryInfo"))
							{
								// The Query was successful, so allocate the memory.
								dwFileSize = (DWORD)atol(bufQuery) ;
							}
							else
							{
								// The Query failed. Allocate some memory. Should allocate memory in blocks.
								dwFileSize = 5*1024 ;
							}


							dinfo->data = new char[dwFileSize+1024] ;
							DWORD dwBytesRead ;
							BOOL bRead = ::InternetReadFile(dinfo->hHttpFile,	
													dinfo->data,	
													dwFileSize+1, 	
													&dwBytesRead);	
							dinfo->data[dwFileSize] = 0;
							if (Succeeded((HINTERNET)bRead, "InternetReadFile"))
							{
								dinfo->data[dwBytesRead] = 0 ;
							} // InternetReadFile
							else
							{
								if(dinfo->data)
									delete dinfo->data;
								dinfo->data = 0;
							}
						}
					}
				}
			} // HttpSendRequest

			{
				GCriticalSectionLock lock(&dinfo->section);
				::InternetCloseHandle(dinfo->hHttpFile); 
				dinfo->hHttpFile = 0;
			}
		} // HttpOpenRequest

		{
			GCriticalSectionLock lock(&dinfo->section);
			::InternetCloseHandle(dinfo->hConnect) ;
			dinfo->hConnect = 0;
		}
	} // InternetConnect 

end:;
}
catch(...)
{
}
	if( dinfo->monitor )
	{
		dinfo->monitor->enter();
		dinfo->monitor->broadcast();
		dinfo->monitor->leave();
	}

   return;
}